package controllers

import (
	"encoding/json"
	"github.com/kataras/iris"
	"lapp_-wy/utils"
	"lapp_-wy/web/middleware/glog"
	"lapp_-wy/web/middleware/jwts"
	"lapp_-wy/web/models"
	"lapp_-wy/web/supports"
	"time"
)

func InsertCSurveysheetResult(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	//日志
	logs := new(models.LeitServerLog)
	logs.File = "/controllers/Subject_controller.go"
	logs.Level = "info"
	logs.Function = "InsertCSurveysheetResult"
	logs.Message = "添加用户问卷"
	logs.Operator = user.Userid
	logs.TimeStamp = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	logs.InsertRecord()
	data := new(models.CSurveysheetResult)
	data.Lastmodifyby = user.Userid
	data.Createtime = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	data.Lastmodifytime = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	data.Cid = user.Pid
	if err := ctx.ReadJSON(data); err != nil {
		supports.Error(ctx, iris.StatusBadRequest, "json解析错误", nil)
		return
	}
	data.Cid = user.Pid
	err := data.Add()
	if err != nil {
		glog.InfoExt("问题调研", "add err is :", err)
		supports.Error(ctx, iris.StatusBadRequest, "添加失败！", nil)
		return
	}
	supports.Ok(ctx, supports.OptionSuccess, "")
}

func TotalCSurveysheetResult(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	//日志
	logs := new(models.LeitServerLog)
	logs.File = "/controllers/Subject_controller.go"
	logs.Level = "info"
	logs.Function = "TotalCSurveysheetResult"
	logs.Message = "统计"
	logs.Operator = user.Userid
	logs.TimeStamp = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	logs.InsertRecord()
	searchs := ctx.URLParam("searchs")
	var Searchdata models.SearchConditions
	err := json.Unmarshal([]byte(searchs), &Searchdata)
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
		return
	}
	me := new(models.CSurveysheetResult)
	me.Cid = user.Pid
	me.Surveysheetid = ctx.URLParam("surveysheetid")
	data, err := me.SatisfactionTotal(Searchdata)
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
		return
	}
	supports.Ok(ctx, supports.OptionSuccess, data)
}

func TotalMultipleCSurveysheetResult(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	//日志
	logs := new(models.LeitServerLog)
	logs.File = "/controllers/Subject_controller.go"
	logs.Level = "info"
	logs.Function = "TotalCSurveysheetResult"
	logs.Message = "统计"
	logs.Operator = user.Userid
	logs.TimeStamp = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	logs.InsertRecord()
	searchs := ctx.URLParam("searchs")
	var Searchdata models.SearchConditions
	err := json.Unmarshal([]byte(searchs), &Searchdata)
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
		return
	}
	me := new(models.CSurveysheetResult)
	me.Cid = user.Pid
	me.Surveysheetid = ctx.URLParam("surveysheetid")
	data, err := me.SatisfactionMultipleTotal(Searchdata)
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
		return
	}
	supports.Ok(ctx, supports.OptionSuccess, data)
}
